"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CollectionMetadata = void 0;
const constants_1 = require("./constants");
class CollectionMetadata {
    entity;
    constructor(entity) {
        this.entity = entity;
    }
    async synchronize(collection) {
        const indexes = await collection.listIndexes().toArray();
        const entityMetadata = Reflect.getMetadata(constants_1.ENTITY_METADATA, this.entity.prototype) || {
            columns: []
        };
        for (const column of entityMetadata.columns) {
            if (column.options.unique) {
                const isIndexAlreadyCreated = indexes.some(index => index.key[column.propertyKey] != null);
                if (isIndexAlreadyCreated) {
                    continue;
                }
                const indexSpecification = {
                    [column.propertyKey]: 1
                };
                await collection.createIndex(indexSpecification, {
                    unique: true
                });
            }
        }
    }
    static collect(entity) {
        if (this.registry.has(entity)) {
            return this.registry.get(entity);
        }
        const metadata = new CollectionMetadata(entity);
        return this.registry.set(entity, metadata).get(entity);
    }
    static registry = new Map();
}
exports.CollectionMetadata = CollectionMetadata;
//# sourceMappingURL=collection-metadata.js.map