"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Collection = void 0;
const promises_1 = require("node:dns/promises");
const mongodb_1 = require("../lib/mongodb");
class Collection extends mongodb_1.MongoCollection {
    constructor(db, name, options) {
        super(db, name, options);
    }
    countDocuments(filter, options) {
        return super.countDocuments(filter, options);
    }
    upsert(filter, update, options) {
        return super.updateOne(filter, update, {
            upsert: true,
            ...options
        });
    }
    withDocumentLock(filter, handle) {
        return this.lock(filter)
            .then(() => Promise.resolve(handle()))
            .then(async (result) => {
            await this.unlock(filter);
            return result;
        })
            .catch(async (error) => {
            console.log('error:', error);
            if (error.message === 'DOCUMENT_IS_STILL_PINNED') {
                throw error;
            }
            await this.unlock(filter);
            throw error;
        });
    }
    async lock(filter) {
        const lock = {
            _lockPinned: true,
            _lockOwnerHostname: process.env.HOSTNAME || 'localhost',
            _lockOwnerPid: process.pid
        };
        const filters = {
            ...filter,
            _lockPinned: { $in: [undefined, false] }
        };
        const update = {
            $set: lock
        };
        const isLockPinned = await super.findOne({
            ...filter,
            _lockPinned: true
        });
        if (isLockPinned) {
            const pid = isLockPinned._lockOwnerPid;
            const hostname = isLockPinned._lockOwnerHostname;
            if (pid === process.pid) {
                throw new Error('DOCUMENT_IS_STILL_PINNED');
            }
            return (0, promises_1.resolve)(hostname)
                .then(() => {
                throw new Error('DOCUMENT_IS_STILL_PINNED');
            })
                .catch(() => this.unlock(filter));
        }
        return super.updateOne(filters, update, {
            upsert: true
        });
    }
    unlock(filter) {
        const filters = {
            ...filter,
            _lockPinned: true
        };
        const lock = {
            _lockPinned: true,
            _lockOwnerHostname: true,
            _lockOwnerPid: true
        };
        return super.updateOne(filters, {
            $unset: lock
        });
    }
}
exports.Collection = Collection;
//# sourceMappingURL=mongo.collection.js.map